@@ -14,17 +14,15 @@
 
 use std::{marker::PhantomData, ptr::NonNull};
 
-#[derive(Clone, Debug, Eq, PartialEq)]
-pub(crate) enum CacheRegion {
-    Window,
-    MainProbation,
-    MainProtected,
-    WriteOrder,
-}
+use super::CacheRegion;
 
+// `crate::{sync,unsync}::DeqNodes` uses a `tagptr::TagNonNull<DeqNode<T>, 2>`
+// pointer. To reserve the space for the 2-bit tag, use 4 bytes as the *minimum*
+// alignment.
+// https://doc.rust-lang.org/reference/type-layout.html#the-alignment-modifiers
+#[repr(align(4))]
 #[derive(PartialEq, Eq)]
 pub(crate) struct DeqNode<T> {
-    pub(crate) region: CacheRegion,
     next: Option<NonNull<DeqNode<T>>>,
     prev: Option<NonNull<DeqNode<T>>>,
     pub(crate) element: T,
@@ -33,7 +31,6 @@ pub(crate) struct DeqNode<T> {
 impl<T> std::fmt::Debug for DeqNode<T> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("DeqNode")
-            .field("region", &self.region)
             .field("next", &self.next)
             .field("prev", &self.prev)
             .finish()
@@ -41,9 +38,11 @@ impl<T> std::fmt::Debug for DeqNode<T> {
 }
 
 impl<T> DeqNode<T> {
-    pub(crate) fn new(region: CacheRegion, element: T) -> Self {
+    pub(crate) fn new(element: T) -> Self {
+        #[cfg(feature = "unstable-debug-counters")]
+        crate::sync::debug_counters::InternalGlobalDebugCounters::deq_node_created();
+
         Self {
-            region,
             next: None,
             prev: None,
             element,
@@ -55,6 +54,13 @@ impl<T> DeqNode<T> {
     }
 }
 
+#[cfg(feature = "unstable-debug-counters")]
+impl<T> Drop for DeqNode<T> {
+    fn drop(&mut self) {
+        crate::sync::debug_counters::InternalGlobalDebugCounters::deq_node_dropped();
+    }
+}
+
 /// Cursor is used to remember the current iterating position.
 enum DeqCursor<T> {
     Node(NonNull<DeqNode<T>>),
@@ -103,8 +109,8 @@ impl<T> Deque<T> {
         }
     }
 
-    pub(crate) fn region(&self) -> &CacheRegion {
-        &self.region
+    pub(crate) fn region(&self) -> CacheRegion {
+        self.region
     }
 
     pub(crate) fn len(&self) -> usize {
@@ -112,7 +118,7 @@ impl<T> Deque<T> {
     }
 
     pub(crate) fn contains(&self, node: &DeqNode<T>) -> bool {
-        self.region == node.region && (node.prev.is_some() || self.is_head(node))
+        node.prev.is_some() || self.is_head(node)
     }
 
     pub(crate) fn peek_front(&self) -> Option<&DeqNode<T>> {
@@ -219,8 +225,6 @@ impl<T> Deque<T> {
     ///
     /// Panics:
     pub(crate) unsafe fn unlink(&mut self, mut node: NonNull<DeqNode<T>>) {
-        assert_eq!(self.region, node.as_ref().region);
-
         if self.is_at_cursor(node.as_ref()) {
             self.advance_cursor();
         }
@@ -338,7 +342,7 @@ mod tests {
         assert!(deque.peek_back().is_none());
 
         // push_back(node1)
-        let node1 = DeqNode::new(MainProbation, "a".to_string());
+        let node1 = DeqNode::new("a".to_string());
         assert!(!deque.contains(&node1));
         let node1 = Box::new(node1);
         let node1_ptr = deque.push_back(node1);
@@ -346,44 +350,44 @@ mod tests {
 
         // peek_front() -> node1
         let head_a = deque.peek_front().unwrap();
-        assert!(deque.contains(&head_a));
-        assert!(deque.is_head(&head_a));
-        assert!(deque.is_tail(&head_a));
+        assert!(deque.contains(head_a));
+        assert!(deque.is_head(head_a));
+        assert!(deque.is_tail(head_a));
         assert_eq!(head_a.element, "a".to_string());
 
         // move_to_back(node1)
-        unsafe { deque.move_to_back(node1_ptr.clone()) };
+        unsafe { deque.move_to_back(node1_ptr) };
         assert_eq!(deque.len(), 1);
 
         // peek_front() -> node1
         let head_b = deque.peek_front().unwrap();
-        assert!(deque.contains(&head_b));
-        assert!(deque.is_head(&head_b));
-        assert!(deque.is_tail(&head_b));
+        assert!(deque.contains(head_b));
+        assert!(deque.is_head(head_b));
+        assert!(deque.is_tail(head_b));
         assert!(std::ptr::eq(head_b, unsafe { node1_ptr.as_ref() }));
         assert!(head_b.prev.is_none());
         assert!(head_b.next.is_none());
 
         // peek_back() -> node1
         let tail_a = deque.peek_back().unwrap();
-        assert!(deque.contains(&tail_a));
-        assert!(deque.is_head(&tail_a));
-        assert!(deque.is_tail(&tail_a));
+        assert!(deque.contains(tail_a));
+        assert!(deque.is_head(tail_a));
+        assert!(deque.is_tail(tail_a));
         assert!(std::ptr::eq(tail_a, unsafe { node1_ptr.as_ref() }));
         assert!(tail_a.prev.is_none());
         assert!(tail_a.next.is_none());
 
         // push_back(node2)
-        let node2 = DeqNode::new(MainProbation, "b".to_string());
+        let node2 = DeqNode::new("b".to_string());
         assert!(!deque.contains(&node2));
         let node2_ptr = deque.push_back(Box::new(node2));
         assert_eq!(deque.len(), 2);
 
         // peek_front() -> node1
         let head_c = deque.peek_front().unwrap();
-        assert!(deque.contains(&head_c));
-        assert!(deque.is_head(&head_c));
-        assert!(!deque.is_tail(&head_c));
+        assert!(deque.contains(head_c));
+        assert!(deque.is_head(head_c));
+        assert!(!deque.is_tail(head_c));
         assert!(std::ptr::eq(head_c, unsafe { node1_ptr.as_ref() }));
         assert!(head_c.prev.is_none());
         assert!(std::ptr::eq(
@@ -392,14 +396,14 @@ mod tests {
         ));
 
         // move_to_back(node2)
-        unsafe { deque.move_to_back(node2_ptr.clone()) };
+        unsafe { deque.move_to_back(node2_ptr) };
         assert_eq!(deque.len(), 2);
 
         // peek_front() -> node1
         let head_d = deque.peek_front().unwrap();
-        assert!(deque.contains(&head_d));
-        assert!(deque.is_head(&head_d));
-        assert!(!deque.is_tail(&head_d));
+        assert!(deque.contains(head_d));
+        assert!(deque.is_head(head_d));
+        assert!(!deque.is_tail(head_d));
         assert!(std::ptr::eq(head_d, unsafe { node1_ptr.as_ref() }));
         assert!(head_d.prev.is_none());
         assert!(std::ptr::eq(
@@ -409,9 +413,9 @@ mod tests {
 
         // peek_back() -> node2
         let tail_b = deque.peek_back().unwrap();
-        assert!(deque.contains(&tail_b));
-        assert!(!deque.is_head(&tail_b));
-        assert!(deque.is_tail(&tail_b));
+        assert!(deque.contains(tail_b));
+        assert!(!deque.is_head(tail_b));
+        assert!(deque.is_tail(tail_b));
         assert!(std::ptr::eq(tail_b, unsafe { node2_ptr.as_ref() }));
         assert!(std::ptr::eq(
             unsafe { tail_b.prev.unwrap().as_ref() },
@@ -421,14 +425,14 @@ mod tests {
         assert!(tail_b.next.is_none());
 
         // move_to_back(node1)
-        unsafe { deque.move_to_back(node1_ptr.clone()) };
+        unsafe { deque.move_to_back(node1_ptr) };
         assert_eq!(deque.len(), 2);
 
         // peek_front() -> node2
         let head_e = deque.peek_front().unwrap();
-        assert!(deque.contains(&head_e));
-        assert!(deque.is_head(&head_e));
-        assert!(!deque.is_tail(&head_e));
+        assert!(deque.contains(head_e));
+        assert!(deque.is_head(head_e));
+        assert!(!deque.is_tail(head_e));
         assert!(std::ptr::eq(head_e, unsafe { node2_ptr.as_ref() }));
         assert!(head_e.prev.is_none());
         assert!(std::ptr::eq(
@@ -438,9 +442,9 @@ mod tests {
 
         // peek_back() -> node1
         let tail_c = deque.peek_back().unwrap();
-        assert!(deque.contains(&tail_c));
-        assert!(!deque.is_head(&tail_c));
-        assert!(deque.is_tail(&tail_c));
+        assert!(deque.contains(tail_c));
+        assert!(!deque.is_head(tail_c));
+        assert!(deque.is_tail(tail_c));
         assert!(std::ptr::eq(tail_c, unsafe { node1_ptr.as_ref() }));
         assert!(std::ptr::eq(
             unsafe { tail_c.prev.unwrap().as_ref() },
@@ -449,16 +453,16 @@ mod tests {
         assert!(tail_c.next.is_none());
 
         // push_back(node3)
-        let node3 = DeqNode::new(MainProbation, "c".to_string());
+        let node3 = DeqNode::new("c".to_string());
         assert!(!deque.contains(&node3));
         let node3_ptr = deque.push_back(Box::new(node3));
         assert_eq!(deque.len(), 3);
 
         // peek_front() -> node2
         let head_f = deque.peek_front().unwrap();
-        assert!(deque.contains(&head_f));
-        assert!(deque.is_head(&head_f));
-        assert!(!deque.is_tail(&head_f));
+        assert!(deque.contains(head_f));
+        assert!(deque.is_head(head_f));
+        assert!(!deque.is_tail(head_f));
         assert!(std::ptr::eq(head_f, unsafe { node2_ptr.as_ref() }));
         assert!(head_f.prev.is_none());
         assert!(std::ptr::eq(
@@ -470,9 +474,9 @@ mod tests {
         let tail_d = deque.peek_back().unwrap();
         assert!(std::ptr::eq(tail_d, unsafe { node3_ptr.as_ref() }));
         assert_eq!(tail_d.element, "c".to_string());
-        assert!(deque.contains(&tail_d));
-        assert!(!deque.is_head(&tail_d));
-        assert!(deque.is_tail(&tail_d));
+        assert!(deque.contains(tail_d));
+        assert!(!deque.is_head(tail_d));
+        assert!(deque.is_tail(tail_d));
         assert!(std::ptr::eq(tail_d, unsafe { node3_ptr.as_ref() }));
         assert!(std::ptr::eq(
             unsafe { tail_d.prev.unwrap().as_ref() },
@@ -481,14 +485,14 @@ mod tests {
         assert!(tail_d.next.is_none());
 
         // move_to_back(node1)
-        unsafe { deque.move_to_back(node1_ptr.clone()) };
+        unsafe { deque.move_to_back(node1_ptr) };
         assert_eq!(deque.len(), 3);
 
         // peek_front() -> node2
         let head_g = deque.peek_front().unwrap();
-        assert!(deque.contains(&head_g));
-        assert!(deque.is_head(&head_g));
-        assert!(!deque.is_tail(&head_g));
+        assert!(deque.contains(head_g));
+        assert!(deque.is_head(head_g));
+        assert!(!deque.is_tail(head_g));
         assert!(std::ptr::eq(head_g, unsafe { node2_ptr.as_ref() }));
         assert!(head_g.prev.is_none());
         assert!(std::ptr::eq(
@@ -498,9 +502,9 @@ mod tests {
 
         // peek_back() -> node1
         let tail_e = deque.peek_back().unwrap();
-        assert!(deque.contains(&tail_e));
-        assert!(!deque.is_head(&tail_e));
-        assert!(deque.is_tail(&tail_e));
+        assert!(deque.contains(tail_e));
+        assert!(!deque.is_head(tail_e));
+        assert!(deque.is_tail(tail_e));
         assert!(std::ptr::eq(tail_e, unsafe { node1_ptr.as_ref() }));
         assert!(std::ptr::eq(
             unsafe { tail_e.prev.unwrap().as_ref() },
@@ -519,9 +523,9 @@ mod tests {
 
         // peek_front() -> node2
         let head_h = deque.peek_front().unwrap();
-        assert!(deque.contains(&head_h));
-        assert!(deque.is_head(&head_h));
-        assert!(!deque.is_tail(&head_h));
+        assert!(deque.contains(head_h));
+        assert!(deque.is_head(head_h));
+        assert!(!deque.is_tail(head_h));
         assert!(std::ptr::eq(head_h, unsafe { node2_ptr.as_ref() }));
         assert!(head_h.prev.is_none());
         assert!(std::ptr::eq(
@@ -531,9 +535,9 @@ mod tests {
 
         // peek_back() -> node1
         let tail_f = deque.peek_back().unwrap();
-        assert!(deque.contains(&tail_f));
-        assert!(!deque.is_head(&tail_f));
-        assert!(deque.is_tail(&tail_f));
+        assert!(deque.contains(tail_f));
+        assert!(!deque.is_head(tail_f));
+        assert!(deque.is_tail(tail_f));
         assert!(std::ptr::eq(tail_f, unsafe { node1_ptr.as_ref() }));
         assert!(std::ptr::eq(
             unsafe { tail_f.prev.unwrap().as_ref() },
@@ -552,18 +556,18 @@ mod tests {
 
         // peek_front() -> node1
         let head_g = deque.peek_front().unwrap();
-        assert!(deque.contains(&head_g));
-        assert!(deque.is_head(&head_g));
-        assert!(deque.is_tail(&head_g));
+        assert!(deque.contains(head_g));
+        assert!(deque.is_head(head_g));
+        assert!(deque.is_tail(head_g));
         assert!(std::ptr::eq(head_g, unsafe { node1_ptr.as_ref() }));
         assert!(head_g.prev.is_none());
         assert!(head_g.next.is_none());
 
         // peek_back() -> node1
         let tail_g = deque.peek_back().unwrap();
-        assert!(deque.contains(&tail_g));
-        assert!(deque.is_head(&tail_g));
-        assert!(deque.is_tail(&tail_g));
+        assert!(deque.contains(tail_g));
+        assert!(deque.is_head(tail_g));
+        assert!(deque.is_tail(tail_g));
         assert!(std::ptr::eq(tail_g, unsafe { node1_ptr.as_ref() }));
         assert!(tail_g.next.is_none());
         assert!(tail_g.next.is_none());
@@ -591,11 +595,11 @@ mod tests {
         let mut deque: Deque<String> = Deque::new(MainProbation);
         assert!((&mut deque).next().is_none());
 
-        let node1 = DeqNode::new(MainProbation, "a".into());
+        let node1 = DeqNode::new("a".into());
         deque.push_back(Box::new(node1));
-        let node2 = DeqNode::new(MainProbation, "b".into());
+        let node2 = DeqNode::new("b".into());
         let node2_ptr = deque.push_back(Box::new(node2));
-        let node3 = DeqNode::new(MainProbation, "c".into());
+        let node3 = DeqNode::new("c".into());
         let node3_ptr = deque.push_back(Box::new(node3));
 
         // -------------------------------------------------------
@@ -643,7 +647,7 @@ mod tests {
 
         // -------------------------------------------------------
         // Try pop_front during iteration.
-        let node3 = DeqNode::new(MainProbation, "c".into());
+        let node3 = DeqNode::new("c".into());
         deque.push_back(Box::new(node3));
 
         assert_eq!((&mut deque).next(), Some(&"a".into()));
@@ -665,11 +669,11 @@ mod tests {
     fn next_node() {
         let mut deque: Deque<String> = Deque::new(MainProbation);
 
-        let node1 = DeqNode::new(MainProbation, "a".into());
+        let node1 = DeqNode::new("a".into());
         deque.push_back(Box::new(node1));
-        let node2 = DeqNode::new(MainProbation, "b".into());
+        let node2 = DeqNode::new("b".into());
         let node2_ptr = deque.push_back(Box::new(node2));
-        let node3 = DeqNode::new(MainProbation, "c".into());
+        let node3 = DeqNode::new("c".into());
         let node3_ptr = deque.push_back(Box::new(node3));
 
         // -------------------------------------------------------
@@ -721,10 +725,10 @@ mod tests {
         let mut deque: Deque<X> = Deque::new(MainProbation);
         let dropped = Rc::new(RefCell::new(Vec::default()));
 
-        let node1 = DeqNode::new(MainProbation, X(1, Rc::clone(&dropped)));
-        let node2 = DeqNode::new(MainProbation, X(2, Rc::clone(&dropped)));
-        let node3 = DeqNode::new(MainProbation, X(3, Rc::clone(&dropped)));
-        let node4 = DeqNode::new(MainProbation, X(4, Rc::clone(&dropped)));
+        let node1 = DeqNode::new(X(1, Rc::clone(&dropped)));
+        let node2 = DeqNode::new(X(2, Rc::clone(&dropped)));
+        let node3 = DeqNode::new(X(3, Rc::clone(&dropped)));
+        let node4 = DeqNode::new(X(4, Rc::clone(&dropped)));
         deque.push_back(Box::new(node1));
         deque.push_back(Box::new(node2));
         deque.push_back(Box::new(node3));
