@@ -1,10 +1,13 @@
 use super::{deques::Deques, AccessTime, CacheBuilder, KeyDate, KeyHashDate, ValueEntry, Weigher};
-use crate::common::{
-    self,
-    deque::{DeqNode, Deque},
-    frequency_sketch::FrequencySketch,
-    time::{CheckedTimeOps, Clock, Instant},
-    CacheRegion,
+use crate::{
+    common::{
+        self,
+        deque::{DeqNode, Deque},
+        frequency_sketch::FrequencySketch,
+        time::{CheckedTimeOps, Clock, Instant},
+        CacheRegion,
+    },
+    Policy,
 };
 
 use smallvec::SmallVec;
@@ -23,10 +26,12 @@ type CacheStore<K, V, S> = std::collections::HashMap<Rc<K>, ValueEntry<K, V>, S>
 
 /// An in-memory cache that is _not_ thread-safe.
 ///
-/// `Cache` utilizes a hash table `std::collections::HashMap` from the standard
-/// library for the central key-value storage. `Cache` performs a best-effort
-/// bounding of the map using an entry replacement algorithm to determine which
-/// entries to evict when the capacity is exceeded.
+/// `Cache` utilizes a hash table [`std::collections::HashMap`][std-hashmap] from the
+/// standard library for the central key-value storage. `Cache` performs a
+/// best-effort bounding of the map using an entry replacement algorithm to determine
+/// which entries to evict when the capacity is exceeded.
+///
+/// [std-hashmap]: https://doc.rust-lang.org/std/collections/struct.HashMap.html
 ///
 /// # Characteristic difference between `unsync` and `sync`/`future` caches
 ///
@@ -41,7 +46,7 @@ type CacheStore<K, V, S> = std::collections::HashMap<Rc<K>, ValueEntry<K, V>, S>
 ///
 /// # Examples
 ///
-/// Cache entries are manually added using an insert method, and are stored in the
+/// Cache entries are manually added using the insert method, and are stored in the
 /// cache until either evicted or manually invalidated.
 ///
 /// Here's an example of reading and updating a cache by using the main thread:
@@ -359,19 +364,12 @@ where
         self.saturating_sub_from_total_weight(invalidated);
     }
 
-    /// Returns the `max_capacity` of this cache.
-    pub fn max_capacity(&self) -> Option<usize> {
-        self.max_capacity.map(|n| n as usize)
-    }
-
-    /// Returns the `time_to_live` of this cache.
-    pub fn time_to_live(&self) -> Option<Duration> {
-        self.time_to_live
-    }
-
-    /// Returns the `time_to_idle` of this cache.
-    pub fn time_to_idle(&self) -> Option<Duration> {
-        self.time_to_idle
+    /// Returns a read-only cache policy of this cache.
+    ///
+    /// At this time, cache policy cannot be modified after cache creation.
+    /// A future version may support to modify it.
+    pub fn policy(&self) -> Policy {
+        Policy::new(self.max_capacity, 1, self.time_to_live, self.time_to_idle)
     }
 }
 
@@ -930,7 +928,7 @@ fn weigh<K, V>(weigher: &mut Option<Weigher<K, V>>, key: &K, value: &V) -> u32 {
 #[cfg(test)]
 mod tests {
     use super::Cache;
-    use crate::{common::time::Clock, unsync::CacheBuilder};
+    use crate::common::time::Clock;
 
     use std::time::Duration;
 
@@ -1107,7 +1105,8 @@ mod tests {
 
     #[test]
     fn time_to_live() {
-        let mut cache = CacheBuilder::new(100)
+        let mut cache = Cache::builder()
+            .max_capacity(100)
             .time_to_live(Duration::from_secs(10))
             .build();
         cache.enable_frequency_sketch_for_testing();
@@ -1151,7 +1150,8 @@ mod tests {
 
     #[test]
     fn time_to_idle() {
-        let mut cache = CacheBuilder::new(100)
+        let mut cache = Cache::builder()
+            .max_capacity(100)
             .time_to_idle(Duration::from_secs(10))
             .build();
         cache.enable_frequency_sketch_for_testing();
